package com.zhentao.common.user.service.impl;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.zhentao.common.user.domain.GooseUser;
import com.zhentao.common.user.dto.LoginDto;
import com.zhentao.common.user.mapper.GooseUserMapper;
import com.zhentao.common.user.service.GooseUserService;
import com.zhentao.util.HttpUtils;
import com.zhentao.util.ResultVo;
import com.zhentao.util.SnowflakeIdGenerator;
import com.zhentao.util.TokenUtils;
import lombok.SneakyThrows;
import org.apache.http.HttpResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.RestTemplate;
import redis.clients.jedis.Jedis;

import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * @author 31810
 * @description 针对表【goose_user】的数据库操作Service实现
 * @createDate 2025-05-06 12:01:03
 */
@Service
public class GooseUserServiceImpl extends ServiceImpl<GooseUserMapper, GooseUser>
        implements GooseUserService {
    @Autowired
    private GooseUserMapper userMapper;
    @Value("${wechat.appid}")
    private String APPID;

    @Value("${wechat.secret}")
    private String SECRET;
    private static final ObjectMapper objectMapper = new ObjectMapper(); // JSON 解析器
    @Override
    public ResultVo login(LoginDto loginDto) {
        if(StringUtils.checkValNotNull(loginDto.getPhone())){
            QueryWrapper<GooseUser> queryWrapper=new QueryWrapper<>();
            queryWrapper.eq("phone",loginDto.getPhone());
            GooseUser user = userMapper.selectOne(queryWrapper);
            if(user==null){
                GooseUser gooseUser1=new GooseUser();
                gooseUser1.setPhone(loginDto.getPhone());

                if(StringUtils.checkValNotNull(loginDto.getPassword())){
                    gooseUser1.setId(SnowflakeIdGenerator.getSnowId());
                    gooseUser1.setNickname("普通用户"+ UUID.randomUUID().toString().replace("-","").substring(0,8));
                    String password = gooseUser1.getPassword();
                    String salt=UUID.randomUUID().toString().replace(",","");
                    String pwd= DigestUtils.md5DigestAsHex((password+salt).getBytes(StandardCharsets.UTF_8));
                    gooseUser1.setPassword(pwd);
                    gooseUser1.setSalt(salt);
                    userMapper.insert(gooseUser1);
                    String token= TokenUtils.createJwtToken(gooseUser1.getId().toString());
                    return ResultVo.OK(token);
                }
                if(StringUtils.checkValNotNull(loginDto.getCode())){
                    gooseUser1.setId(SnowflakeIdGenerator.getSnowId());
                    gooseUser1.setNickname("普通用户"+ UUID.randomUUID().toString().replace("-","").substring(0,8));
                    Jedis jedis=new Jedis("localhost");
                    String code = jedis.get("code");
                    if(loginDto.getCode().equals(code)){
                        userMapper.insert(gooseUser1);
                        String token= TokenUtils.createJwtToken(gooseUser1.getId().toString());
                        return ResultVo.OK(token);
                    }
                }
                return ResultVo.ERROR();
            }
            String token= TokenUtils.createJwtToken(user.getId().toString());
            return ResultVo.OK(token);
        }
        return ResultVo.ERROR();
    }

    @SneakyThrows
    @Override
    public ResultVo Wxlogin(Map<String, String> requestData) {
        String code = requestData.get("code");
        System.out.println(code);
        if (code == null) {
            return new ResultVo(400,"Missing code",null);
        }
        String url = "https://api.weixin.qq.com/sns/jscode2session?" +
                "appid=" + APPID +
                "&secret=" + SECRET +
                "&js_code=" + code +
                "&grant_type=authorization_code";
        RestTemplate restTemplate = new RestTemplate();
        HttpHeaders headers = new HttpHeaders();
        HttpEntity<String> entity = new HttpEntity<>(headers);

        try {
            // 改用 String 接收响应，兼容所有 Content-Type
            ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.GET, entity, String.class);
            String responseBody = response.getBody();

            System.out.println(responseBody);

            // 手动解析 JSON
            Map<String, Object> result = objectMapper.readValue(responseBody, Map.class);
            System.out.println(result);
            if (result.containsKey("errcode")) { // 微信错误响应
                return new ResultVo(400,"微信接口返回错误: " + result.get("errmsg"),null);
            }
            // 成功响应
            QueryWrapper<GooseUser>queryWrapper=new QueryWrapper<>();
            String openId=(String) result.get("openid");
            String session_key=(String) result.get("session_key");
            queryWrapper.eq("open_id",openId);
            GooseUser gooseUser = userMapper.selectOne(queryWrapper);
            if(gooseUser==null){
                GooseUser user=new GooseUser();
                user.setId(SnowflakeIdGenerator.getSnowId());
                user.setOpenId(openId);
                user.setNickname("微信用户"+ UUID.randomUUID().toString().replace("-","").substring(0,8));
                userMapper.insert(user);
                String token=TokenUtils.createJwtToken(user.getId().toString());
                return new ResultVo(200,"登录成功",token);
            }
            String token=TokenUtils.createJwtToken(gooseUser.getId().toString());
            return new ResultVo(200,"登录成功",token);
        } catch (HttpClientErrorException e) { // 处理 HTTP 错误（如 400、500）
            return new ResultVo(400,"请求微信接口失败: " + e.getResponseBodyAsString(),null);
        } catch (Exception e) { // 处理 JSON 解析错误
            return new ResultVo(400,"响应解析失败: " + e.getMessage(),null);
        }
    }

    @Override
    public ResultVo getCode(GooseUser user) {
        String host = "https://gyytz.market.alicloudapi.com";
        String path = "/sms/smsSend";
        String method = "POST";
        String appcode = "b685e5e231ce404c855db67359acb1e1";
        Map<String, String> headers = new HashMap<String, String>();
        // 最后在 header 中的格式(中间是英文空格)为 Authorization:APPCODE 83359fd73fe94948385f570e3c139105
        headers.put("Authorization", "APPCODE " + appcode);
        Map<String, String> querys = new HashMap<String, String>();
        System.out.println(user.getPhone());
        querys.put("mobile",user.getPhone());
        long uuidLong = UUID.randomUUID().getMostSignificantBits() ^ UUID.randomUUID().getLeastSignificantBits();

        // 2. 转换为正整数并取模1000000（确保6位范围）
        long code = Math.abs(uuidLong) % 1000000;

        String code1=String.format("%06d", code);
        System.out.println(code1);
        if(code1!=null){
            Jedis jedis=new Jedis("localhost",6379);
            jedis.set("code",code1);
        }

        querys.put("param", "**code**:"+code1+",**minute**:5");
        // smsSignId（短信前缀）和 templateId（短信模板），可登录国阳云控制台自助申请。参考文档：http://help.guoyangyun.com/Problem/Qm.html
        querys.put("smsSignId", "2e65b1bb3d054466b82f0c9d125465e2");
        querys.put("templateId", "908e94ccf08b4476ba6c876d13f084ad");
        Map<String, String> bodys = new HashMap<String, String>();
        try {
            /**
             * 重要提示如下:
             * HttpUtils 请从
             * https://github.com/aliyun/api-gateway-demo-sign-java/blob/master/src/main/java/com/aliyun/api/gateway/demo/util/HttpUtils.java
             * 下载
             *
             * 相应的依赖请参照
             * https://github.com/aliyun/api-gateway-demo-sign-java/blob/master/pom.xml
             */
            HttpResponse response = HttpUtils.doPost(host, path, method, headers, querys, bodys);
            System.out.println(response.toString());
            // 获取 response 的 body
            // System.out.println(EntityUtils.toString(response.getEntity()));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    // 构建成功响应
    private Map<String, Object> buildSuccessResponse(String openid, String sessionKey) {
        Map<String, Object> response = new HashMap<>();
        System.out.println("openid:"+openid);
        System.out.println("sessionKey:"+sessionKey);
        response.put("openid", openid);
        response.put("session_key", sessionKey);
        return response;
    }

    // 构建错误响应
    private Map<String, Object> buildErrorResponse(String message) {
        Map<String, Object> response = new HashMap<>();
        response.put("error", message);
        return response;
    }
}




